# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2018-2025, by Samuel Williams.

require_relative "covered/version"
require_relative "covered/policy"

require_relative "covered/brief_summary"
require_relative "covered/partial_summary"
